﻿using COM;
using Microsoft.Win32;
using RGiesecke.DllExport;
using System;
using System.Runtime.InteropServices;

namespace MyCOMObj
{
    public class Register
    {

        [DllExport("DllRegisterServer", CallingConvention = CallingConvention.StdCall)]
        public static int DllRegisterServer()
        {
            try
            {
                if (IntPtr.Size == 4)
                {
                    RegisterDLL(RegistryView.Registry32);
                }
                else
                {
                    RegisterDLL(RegistryView.Registry64);
                }
            }
            catch (Exception e)
            {
                return Marshal.GetHRForException(e);
            }

            return 0; // S_OK
        }

        private static void RegisterDLL(RegistryView regView)
        {
            using (RegistryKey clsRoot = RegistryKey.OpenBaseKey(RegistryHive.ClassesRoot, regView))
            {
                using (RegistryKey clsKey = clsRoot.OpenSubKey("CLSID", true))
                using (RegistryKey clsIdKey = clsKey.CreateSubKey(MyClassFactory.CLSID_KEY))
                {
                    using (RegistryKey inprocKey = clsIdKey.CreateSubKey("InProcServer32"))
                    {
                        inprocKey.SetValue(null, typeof(Register).Assembly.Location);
                        inprocKey.SetValue("ThreadingModel", "Apartment");
                    }
                }
            }
        }

        [DllExport("DllUnregisterServer", CallingConvention = CallingConvention.StdCall)]
        public static int DllUnregisterServer()
        {
            try
            {
                if (IntPtr.Size == 4)
                {
                    UnregisterDLL(RegistryView.Registry32);
                }
                else
                {
                    UnregisterDLL(RegistryView.Registry64);
                }
            }
            catch (Exception e)
            {
                return Marshal.GetHRForException(e);
            }

            return 0; // S_OK
        }

        private static void UnregisterDLL(RegistryView regView)
        {
            using (RegistryKey clsRoot = RegistryKey.OpenBaseKey(RegistryHive.ClassesRoot, regView))
            {
                using (RegistryKey clsKey = clsRoot.OpenSubKey("CLSID", true))
                {
                    clsKey.DeleteSubKeyTree(MyClassFactory.CLSID_KEY, false);
                }
            }
        }

        [DllExport("DllCanUnloadNow", CallingConvention = CallingConvention.StdCall)]
        public static int DllCanUnloadNow()
        {
            return -1; // S_FALSE
        }

        [DllExport("DllGetClassObject", CallingConvention = CallingConvention.StdCall)]
        public static int DllGetClassObject(ref Guid rclsid, ref Guid riid, out IntPtr pUnk)
        {
            pUnk = IntPtr.Zero;

            if (rclsid.ToString().ToUpper() == MyClassFactory.CLSID)
            {
                if (riid.ToString().ToUpper() == Guids.IClassFactory)
                {
                    pUnk = Marshal.GetComInterfaceForObject(new MyClassFactory(), typeof(IClassFactory));
                    return 0; // S_OK
                }
            }

            return -1; // S_FALSE
        }
    }
}
